@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2010-2012, 2014-2016 Free Software
-   Foundation, Inc.
+   Copyright (C) 2005-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,7 +29,7 @@ enumfun (mu_folder_t folder, struct mu_list_response *resp, void *data)
 	  (resp->type & MU_FOLDER_ATTRIBUTE_DIRECTORY) ? 'd' : '-',
 	  (resp->type & MU_FOLDER_ATTRIBUTE_FILE) ? 'f' : '-',
 	  resp->separator,
-	  resp->level,
+	  resp->depth,
 	  resp->name);
   return 0;
 }
