@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2004-2005, 2007-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -490,7 +489,6 @@ mu_header_destroy (mu_header_t *ph)
       *ph = NULL;
     }
 }
-
 
 int
 mu_header_set_value (mu_header_t header, const char *fn, const char *fv,
@@ -511,9 +509,10 @@ mu_header_set_value (mu_header_t header, const char *fn, const char *fv,
   if (fv == NULL && !replace)
     return EINVAL;
 
+  ent = mu_hdrent_find (header, fn, 1);
+  
   if (replace)
     {
-      ent = mu_hdrent_find (header, fn, 1);
       if (ent)
 	{
 	  if (fv == NULL)
@@ -530,12 +529,16 @@ mu_header_set_value (mu_header_t header, const char *fn, const char *fv,
       else if (fv == NULL)
 	return 0;
     }
-
+  else if (ent)
+    {
+      return MU_ERR_EXISTS;
+    }
+  
   ent = mu_hdrent_create (header, NULL,
 			  fn, strlen (fn), fv, strlen (fv));
   if (!ent)
     return ENOMEM;
-  mu_hdrent_prepend (header, ent);
+  mu_hdrent_append (header, ent);
   HEADER_SET_MODIFIED (header);
   return 0;
 }
@@ -685,8 +688,8 @@ mu_header_sget_value_n (mu_header_t header,
 
 int
 mu_header_aget_value_n (mu_header_t header,
-		      const char *name, int n,
-		      char **pval)
+			const char *name, int n,
+			char **pval)
 {
   const char *s;
   int status = mu_header_sget_value_n (header, name, n, &s);
