@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003-2004, 2007-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -390,6 +389,9 @@ _tls_ioctl (struct _mu_stream *stream, int code, int opcode, void *arg)
 	}
       break;
       
+    case MU_IOCTL_TCPSTREAM:
+      return mu_stream_ioctl (sp->transport[0], code, opcode, arg);
+			      
     default:
       return ENOSYS;
     }
