@@ -27,6 +27,7 @@ mail_delete_msg (msgset_t *mspec, mu_message_t msg, void *data)
 
   mu_message_get_attribute (msg, &attr);
   mu_attribute_set_deleted (attr);
+  mu_attribute_unset_userflag (attr, MAIL_ATTRIBUTE_PRESERVED);
   cond_page_invalidate (mspec->msg_part[0]);
   return 0;
 }
@@ -37,7 +38,7 @@ mail_delete (int argc, char **argv)
   int rc = util_foreach_msg (argc, argv, MSG_NODELETED|MSG_SILENT,
 			     mail_delete_msg, NULL);
 
-  if (mailvar_is_true ("autoprint"))
+  if (mailvar_is_true (mailvar_name_autoprint))
     util_do_command("print");
 
   return rc;
