@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2002, 2005, 2007, 2010-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +25,7 @@ mail_folders (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
 {
   char *path;
 
-  if (mailvar_get (&path, "folder", mailvar_type_string, 1))
+  if (mailvar_get (&path, mailvar_name_folder, mailvar_type_string, 1))
     return 1;
 
   if (path[0] != '/' && path[0] != '~')
