@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2007, 2009-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,7 +42,7 @@ util_do_command (const char *fmt, ...)
   char *cmd = NULL;
   size_t size = 0;
   va_list ap;
-  
+
   va_start (ap, fmt);
   status = mu_vasnprintf (&cmd, &size, fmt, ap);
   va_end (ap);
@@ -62,7 +61,7 @@ util_do_command (const char *fmt, ...)
       if (cmd[0] == '\0')
 	{
 	  free (cmd);
-	  
+
 	  /* Hitting return i.e. no command, is equivalent to next
 	     according to the POSIX spec. Note, that this applies
 	     to interactive state only. */
@@ -76,8 +75,9 @@ util_do_command (const char *fmt, ...)
 			 for expansion (see below) */
       if (mu_wordsplit (cmd, &ws, MU_WRDSF_DEFFLAGS|MU_WRDSF_DOOFFS))
 	{
-	  mu_error (_("%s failed: %s"), "mu_wordsplit",
-		    mu_wordsplit_strerror (&ws));
+	  mu_error ("\"%s\": %s", cmd, mu_wordsplit_strerror (&ws));
+	  free (cmd);
+	  return MU_ERR_PARSE;
 	}
       else
 	{
@@ -85,11 +85,11 @@ util_do_command (const char *fmt, ...)
 
 	  argc = ws.ws_wordc;
 	  argv = ws.ws_wordv + 1;
-	  
+
 	  /* Special case: a number alone implies "print" */
 	  if (argc == 1
 	      && ((strtoul (argv[0], &p, 10) > 0 && *p == 0)
-		  || (argv[0][1] == 0 && strchr("^$", argv[0][0]))))
+		  || (argv[0][1] == 0 && strchr ("^$", argv[0][0]))))
 	    {
 	      /* Use the extra slot for "print" command */
 	      argc++;
@@ -98,26 +98,26 @@ util_do_command (const char *fmt, ...)
 	    }
 
 	  entry = mail_find_command (argv[0]);
+	  free (cmd);
 	}
-      free (cmd);
     }
   else
-    entry = mail_find_command ("quit");
+    entry = mail_find_command (mailvar_name_quit);
 
   if (!entry)
     {
       /* argv[0] might be a traditional /bin/mail contracted form, e.g.
 	 `d*' or `p4'. */
-	 
+
       char *p;
-      
+
       for (p = argv[0] + strlen (argv[0]) - 1;
 	   p > argv[0] && !mu_isalpha (*p);
 	   p--)
 	;
 
       p++;
-      
+
       if (strlen (p))
 	{
 	  /* Expand contracted form.  That's what we have kept an extra
@@ -131,10 +131,10 @@ util_do_command (const char *fmt, ...)
 	  ws.ws_wordc++;
 	  ws.ws_offs = 0;
 	}
-      
+
       entry = mail_find_command (argv[0]);
     }
-  
+
   if (entry)
     {
       /* Make sure we are not in any if/else */
@@ -183,12 +183,12 @@ util_foreach_msg (int argc, char **argv, int flags,
 }
 
 size_t
-util_range_msg (size_t low, size_t high, int flags, 
+util_range_msg (size_t low, size_t high, int flags,
 		msg_handler_t func, void *data)
 {
   msgset_t msgspec = { 0 };
   size_t count, expect_count;
-  
+
   msgspec.next = NULL;
   msgspec.npart = 0;
   msgspec.msg_part = &low;
@@ -217,7 +217,7 @@ util_range_msg (size_t low, size_t high, int flags,
 		       (unsigned long) low);
 	 continue;
        }
-     
+
      if (util_get_message (mbox, low, &mesg) == 0)
        {
 	 count ++;
@@ -250,13 +250,13 @@ util_find_entry (void *table, size_t nmemb, size_t size, const char *cmd)
   int i;
   int len = strlen (cmd);
   char *p;
-  
+
   for (p = table, i = 0; i < nmemb; i++, p += size)
     {
       struct mail_command *cp = (struct mail_command *)p;
       int ll = strlen (cp->longname);
       int sl = strlen (cp->shortname);
-      
+
       if (sl > ll && !strncmp (cp->shortname, cmd, sl))
 	return p;
       else if (sl == len && !strcmp (cp->shortname, cmd))
@@ -285,7 +285,7 @@ util_help (void *table, size_t nmemb, size_t size, const char *word)
 	    continue;
 	  mu_stream_printf (out, "%s\n", cp->synopsis);
 	}
-      
+
       mu_stream_unref (out);
 
       return 0;
@@ -312,8 +312,8 @@ util_command_list (void *table, size_t nmemb, size_t size)
   int i;
   char *p;
   int cols = util_screen_columns ();
-  int pos;
-  
+  int pos = 0;
+
   for (p = table, i = 0; i < nmemb; i++, p += size)
     {
       const char *cmd;
@@ -335,7 +335,7 @@ util_command_list (void *table, size_t nmemb, size_t size)
 	  mu_printf ("\n%s ", cmd);
 	}
       else
-        mu_printf ("%s ", cmd);
+	mu_printf ("%s ", cmd);
     }
   mu_printf ("\n");
   return 0;
@@ -376,7 +376,7 @@ util_screen_lines ()
   int screen;
   size_t n;
 
-  if (mailvar_get (&screen, "screen", mailvar_type_number, 0) == 0)
+  if (mailvar_get (&screen, mailvar_name_screen, mailvar_type_number, 0) == 0)
     return screen;
   n = util_getlines();
   util_do_command ("set screen=%lu", (unsigned long) n);
@@ -389,7 +389,7 @@ util_screen_columns ()
   int cols;
   size_t n;
 
-  if (mailvar_get (&cols, "columns", mailvar_type_number, 0) == 0)
+  if (mailvar_get (&cols, mailvar_name_columns, mailvar_type_number, 0) == 0)
     return cols;
   n = util_getcols();
   util_do_command ("set columns=%lu", (unsigned long) n);
@@ -401,9 +401,9 @@ util_get_crt ()
 {
   int lines;
 
-  if (mailvar_get (&lines, "crt", mailvar_type_number, 0) == 0)
+  if (mailvar_get (&lines, mailvar_name_crt, mailvar_type_number, 0) == 0)
     return lines;
-  else if (mailvar_is_true ("crt"))
+  else if (mailvar_is_true (mailvar_name_crt))
     return util_getlines ();
   return 0;
 }
@@ -412,7 +412,7 @@ const char *
 util_reply_prefix ()
 {
   char *prefix = "Re: ";
-  mailvar_get (&prefix, "replyprefix", mailvar_type_string, 0);
+  mailvar_get (&prefix, mailvar_name_replyprefix, mailvar_type_string, 0);
   return prefix;
 }
 
@@ -467,35 +467,21 @@ char *
 util_folder_path (const char *name)
 {
   char *folder;
-  char *tmp;
-  char *p;
-  
-  if (mailvar_get (&folder, "folder", mailvar_type_string, 1))
+  int rc;
+
+  if (mailvar_get (&folder, mailvar_name_folder, mailvar_type_string, 1))
     return NULL;
-      
+
   if (!name)
     return NULL;
-  if (name[0] == '+')
-    name++;
-  
-  if (folder[0] != '/' && folder[0] != '~')
+
+  rc = mu_mailbox_expand_name (name, &folder);
+  if (rc)
     {
-      char *home = mu_get_homedir ();
-      tmp  = mu_alloc (strlen (home) + 1 +
-		       strlen (folder) + 1 +
-		       strlen (name) + 1);
-      sprintf (tmp, "%s/%s/%s", home, folder, name);
+      mu_diag_funcall (MU_DIAG_ERROR, "mailbox_expand_name", name, rc);
+      return NULL;
     }
-  else
-    {
-      tmp  = mu_alloc (strlen (folder) + 1 +
-		       strlen (name) + 1);
-      sprintf (tmp, "%s/%s", folder, name);
-    }
-  p = util_fullpath (tmp);
-  free (tmp);
-  
-  return p;
+  return folder;
 }
 
 char *
@@ -520,7 +506,7 @@ util_get_sender (int msgno, int strip)
 	}
     }
 
-  if (mu_address_aget_email (addr, 1, &buf))
+  if (mu_address_aget_email (addr, 1, &buf) || !buf)
     {
       mu_error (_("Cannot determine sender name (msg %d)"), msgno);
       mu_address_destroy (&addr);
@@ -547,7 +533,8 @@ util_slist_print (mu_list_t list, int nl)
   if (!list || mu_list_get_iterator (list, &itr))
     return;
 
-  for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
+  for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+       mu_iterator_next (itr))
     {
       mu_iterator_current (itr, (void **)&name);
       mu_printf ("%s%c", name, nl ? '\n' : ' ');
@@ -565,7 +552,8 @@ util_slist_lookup (mu_list_t list, const char *str)
   if (!list || mu_list_get_iterator (list, &itr))
     return 0;
 
-  for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
+  for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+       mu_iterator_next (itr))
     {
       mu_iterator_current (itr, (void **)&name);
       if (mu_c_strcasecmp (name, str) == 0)
@@ -659,34 +647,46 @@ char *
 util_outfolder_name (char *str)
 {
   char *outfolder;
+  char *exp;
+  int rc;
 
   if (!str)
     return NULL;
-  
+
   switch (*str)
     {
     case '/':
     case '~':
-      str = util_fullpath (str);
-      break;
-      
     case '+':
-      str = util_folder_path (str);
+      rc = mu_mailbox_expand_name (str, &exp);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_expand_name", str, rc);
+	  return NULL;
+	}
       break;
 
     default:
-      if (mailvar_get (&outfolder, "outfolder", mailvar_type_string, 0) == 0)
+      if (mailvar_get (&outfolder, mailvar_name_outfolder,
+		       mailvar_type_string, 0) == 0)
 	{
-	  char *ns = NULL;
-	  mu_asprintf (&ns, "%s/%s", outfolder, str);
-	  str = util_fullpath (ns);
-	  free (ns);
+	  char *s = mu_make_file_name (outfolder, str);
+	  rc = mu_mailbox_expand_name (s, &exp);
+	  if (rc)
+	    {
+	      mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_expand_name", s, rc);
+	      free (s);
+	      return NULL;
+	    }
+	  free (s);
 	}
+      else
+	exp = mu_strdup (str);
       break;
 
     }
 
-  return mu_strdup (str);
+  return exp;
 }
 
 /* Save an outgoing message. The SAVEFILE argument overrides the setting
@@ -695,8 +695,8 @@ void
 util_save_outgoing (mu_message_t msg, char *savefile)
 {
   char *record;
-  
-  if (mailvar_get (&record, "record", mailvar_type_string, 0) == 0)
+
+  if (mailvar_get (&record, mailvar_name_record, mailvar_type_string, 0) == 0)
     {
       int rc;
       mu_mailbox_t outbox;
@@ -725,7 +725,7 @@ util_save_outgoing (mu_message_t msg, char *savefile)
 
       mu_mailbox_close (outbox);
       mu_mailbox_destroy (&outbox);
-      
+
       free (filename);
     }
 }
@@ -836,7 +836,7 @@ util_merge_addresses (char **addr_str, const char *value)
 
   if ((rc = mu_address_create (&new_addr, value)) != 0)
     return rc;
-      
+
   if ((rc = mu_address_create (&addr, *addr_str)) != 0)
     {
       mu_address_destroy (&new_addr);
@@ -847,10 +847,12 @@ util_merge_addresses (char **addr_str, const char *value)
   if (rc == 0)
     {
       char *val;
-      
+
       rc = mu_address_aget_printable (addr, &val);
       if (rc == 0)
 	{
+	  if (!val)
+	    return MU_ERR_NOENT;
 	  free (*addr_str);
 	  *addr_str = val;
 	}
@@ -871,7 +873,7 @@ is_address_field (const char *name)
     0
   };
   char **p;
-  
+
   for (p = address_fields; *p; p++)
     if (mu_c_strcasecmp (*p, name) == 0)
       return 1;
@@ -884,32 +886,32 @@ util_header_expand (mu_header_t *phdr)
   size_t i, nfields = 0;
   mu_header_t hdr;
   int errcnt = 0, rc;
-  
+
   rc = mu_header_create (&hdr, "", 0);
   if (rc)
     {
       mu_error (_("Cannot create temporary header: %s"), mu_strerror (rc));
       return 1;
     }
-      
+
   mu_header_get_field_count (*phdr, &nfields);
   for (i = 1; i <= nfields; i++)
     {
       const char *name, *value;
-      
+
       if (mu_header_sget_field_name (*phdr, i, &name))
 	continue;
 
       if (mu_header_sget_field_value (*phdr, i, &value))
 	continue;
-      
+
       if (is_address_field (name))
 	{
 	  const char *s;
 	  mu_address_t addr = NULL;
 	  struct mu_wordsplit ws;
 	  size_t j;
-	  
+
 	  if (mu_header_sget_value (hdr, name, &s) == 0)
 	    mu_address_create (&addr, s);
 
@@ -930,9 +932,9 @@ util_header_expand (mu_header_t *phdr)
 	      const char *exp;
 	      mu_address_t new_addr;
 	      char *p = ws.ws_wordv[j];
-	      
-	      if (mailvar_is_true ("inplacealiases"))
-	        /* If inplacealiases was set, the value was already expanded */
+
+	      if (mailvar_is_true (mailvar_name_inplacealiases))
+		/* If inplacealiases was set, the value was already expanded */
 		exp = p;
 	      else
 		exp = alias_expand (p);
@@ -941,23 +943,24 @@ util_header_expand (mu_header_t *phdr)
 		{
 		  errcnt++;
 		  if (exp)
-		    mu_error (_("Cannot parse address `%s' (while expanding `%s'): %s"),
+		    mu_error (_("Cannot parse address `%s'"
+				" (while expanding `%s'): %s"),
 				exp, p, mu_strerror (rc));
 		  else
 		    mu_error (_("Cannot parse address `%s': %s"),
 				p, mu_strerror (rc));
 		}
-	      
+
 	      mu_address_union (&addr, new_addr);
 	      mu_address_destroy (&new_addr);
 	    }
-	  
+
 	  if (addr)
 	    {
 	      const char *newvalue;
-	      
+
 	      rc = mu_address_sget_printable (addr, &newvalue);
-	      if (rc == 0)
+	      if (rc == 0 && newvalue)
 		mu_header_set_value (hdr, name, newvalue, 1);
 	      mu_address_destroy (&addr);
 	    }
@@ -987,7 +990,7 @@ util_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *msg)
       util_error_range (msgno);
       return MU_ERR_NOENT;
     }
-  
+
   status = mu_mailbox_get_message (mbox, msgno, msg);
   if (status)
     {
@@ -1049,7 +1052,7 @@ util_get_charset (void)
 {
   char *charset;
 
-  if (mailvar_get (&charset, "charset", mailvar_type_string, 0))
+  if (mailvar_get (&charset, mailvar_name_charset, mailvar_type_string, 0))
     return NULL;
 
   if (mu_c_strcasecmp (charset, "auto") == 0)
@@ -1058,7 +1061,7 @@ util_get_charset (void)
       char *tmp = getenv ("LC_ALL");
       if (!tmp)
 	tmp = getenv ("LANG");
-      
+
       if (tmp && mu_parse_lc_all (tmp, &lc_all, MU_LC_CSET) == 0)
 	{
 	  charset = mu_strdup (lc_all.charset);
@@ -1087,10 +1090,10 @@ util_rfc2047_decode (char **value)
 
   rc = mu_rfc2047_decode (charset, *value, &tmp);
   free (charset);
-  
+
   if (rc)
     {
-      if (mailvar_is_true ("verbose"))
+      if (mailvar_is_true (mailvar_name_verbose))
 	mu_error (_("Cannot decode line `%s': %s"), *value, mu_strerror (rc));
     }
   else
@@ -1122,7 +1125,7 @@ open_pager (size_t lines)
   const char *pager;
   unsigned pagelines = util_get_crt ();
   mu_stream_t str;
-  
+
   if (pagelines && lines > pagelines && (pager = getenv ("PAGER")))
     {
       int rc = mu_command_stream_create (&str, pager, MU_STREAM_WRITE);
