@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012, 2014-2018 Free Software Foundation,
-   Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1048,9 +1047,15 @@ handle_message (mu_message_t msg, msg_part_t part, msg_handler_t fun, void *data
   if (ismime)
     {
       size_t i, nparts;
-
-      mu_message_get_num_parts (msg, &nparts);
-
+      int rc;
+      
+      rc = mu_message_get_num_parts (msg, &nparts);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERR, "mu_message_get_num_parts", NULL, rc);
+	  return rc;
+	}
+	
       msg_part_incr (part);
       for (i = 1; i <= nparts; i++)
 	{
