@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002-2003, 2005-2012, 2014-2016 Free Software
-   Foundation, Inc.
+   Copyright (C) 2002-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,9 +24,9 @@
 static char prog_doc[] = N_("Reply to a message");
 static char args_doc[] = N_("[MESSAGE]");
 
-static char *format_str = NULL;
 static mh_format_t format;
-static int width = 80;
+static mh_fvm_t fvm;
+static int width;
 
 struct mh_whatnow_env wh_env = { 0 };
 static int initial_edit = 1;
@@ -35,7 +34,6 @@ static const char *whatnowproc;
 static mu_msgset_t msgset;
 static mu_mailbox_t mbox;
 static int build_only = 0; /* -build flag */
-static int nowhatnowproc = 0; /* -nowhatnowproc */
 static int use_draft = 0;  /* -use flag */
 static char *mhl_filter = NULL; /* -filter flag */
 static int annotate;       /* -annotate flag */
@@ -86,7 +84,7 @@ set_whatnowproc (struct mu_parseopt *po, struct mu_option *opt,
 		 char const *arg)
 {
   whatnowproc = mu_strdup (arg);
-  nowhatnowproc = 0;
+  wh_env.nowhatnowproc = 0;
 }
 
 static void
@@ -94,15 +92,11 @@ set_group (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
   if (strcmp (arg, "1") == 0)
     {
-      if (mh_read_formfile ("replgroupcomps", &format_str))
+      if (!format && mh_format_file_parse (&format, "replgroupcomps",
+					   MH_FMT_PARSE_DEFAULT))
 	exit (1);
       rcpt_mask |= RCPT_ALL;
     }
-  else
-    {
-      free (format_str);
-      format_str = NULL;
-    }
 }
 
 static struct mu_option options[] = {
@@ -142,9 +136,9 @@ static struct mu_option options[] = {
   { "filter", 0, N_("MHL-FILTER"), MU_OPTION_DEFAULT,
     N_("set the mhl filter to preprocess the body of the message being replied"),
     mu_c_string, &mhl_filter, mh_opt_find_file },
-  { "form",   0, N_("FILE"), MU_OPTION_DEFAULT,
-    N_("read format from given file") ,
-    mu_c_string, &format_str, mh_opt_read_formfile },
+  { "form",    0, N_("FILE"),   MU_OPTION_DEFAULT,
+    N_("read format from given file"),
+    mu_c_string, &format, mh_opt_parse_formfile },
   { "format", 0, NULL, MU_OPTION_DEFAULT,
     N_("include a copy of the message being replied; the message will be processed using either the default filter \"mhl.reply\", or the filter specified by --filter option"),
     mu_c_string, &mhl_filter, mh_opt_find_file, "mhl.repl" },
@@ -165,7 +159,7 @@ static struct mu_option options[] = {
     mu_c_string, NULL, set_whatnowproc },
   { "nowhatnowproc", 0, NULL, MU_OPTION_DEFAULT,
     N_("don't run whatnowproc"),
-    mu_c_int, &nowhatnowproc, NULL, "1" },
+    mu_c_int, &wh_env.nowhatnowproc, NULL, "1" },
   { "use", 0, NULL, MU_OPTION_DEFAULT,
     N_("use draft file preserved after the last session"),
     mu_c_bool, &use_draft },
@@ -174,39 +168,28 @@ static struct mu_option options[] = {
 };
 
 static char default_format_str[] =
-"%(lit)%(formataddr %<{reply-to}%?{from}%?{sender}%?{return-path}%>)"
-"%<(nonnull)%(void(width))%(putaddr To: )\\n%>"
-"%(lit)%<(rcpt to)%(formataddr{to})%>%<(rcpt cc)%(formataddr{cc})%>%<(rcpt me)%(formataddr(me))%>"
-"%<(nonnull)%(void(width))%(putaddr cc: )\\n%>"
-"%<{fcc}Fcc: %{fcc}\\n%>"
-"%<{subject}Subject: Re: %(unre{subject})\\n%>"
-"%(lit)%(concat(in_reply_to))%<(nonnull)%(void(width))%(printhdr In-reply-to: )\\n%>"
-"%(lit)%(concat(references))%<(nonnull)%(void(width))%(printhdr References: )\\n%>"
-"X-Mailer: MH \\(%(package_string)\\)\\n" 
-"--------\n";
+    "%(lit)%(formataddr %<{reply-to}%?{from}%?{sender}%?{return-path}%>)"
+    "%<(nonnull)%(void(width))%(putaddr To: )\\n%>"
+    "%(lit)%<(rcpt to)%(formataddr{to})%>%<(rcpt cc)%(formataddr{cc})%>%<(rcpt me)%(formataddr(me))%>"
+    "%<(nonnull)%(void(width))%(putaddr cc: )\\n%>"
+    "%<(mymbox{from})%<{fcc}Fcc: %{fcc}\\n%>%>"
+    "Subject: %<{subject}%(putstr %<(profile reply-prefix)%|"
+    "%(void Re:)%>) %(void(unre{subject}))%(trim)%(putstr)%>\n"
+    "%(lit)%<(in_reply_to)%(void(width))%(printhdr In-reply-to: )\\n%>"
+    "%(lit)%<(references)%(void(width))%(printhdr References: )\\n%>"
+    "X-Mailer: MH (%(package_string))\n"
+    "--------\n";
 
 void
 make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
 {
   int rc;
   mu_message_t msg;
-  struct stat st;
   size_t msgno;
   
   /* First check if the draft exists */
-  if (!build_only && stat (wh->draftfile, &st) == 0)
-    {
-      if (use_draft)
-	disp = DISP_USE;
-      else 
-	{
-	  printf (ngettext ("Draft \"%s\" exists (%s byte).\n",
-			    "Draft \"%s\" exists (%s bytes).\n",
-			    (unsigned long) st.st_size),
-		  wh->draftfile, mu_umaxtostr (0, st.st_size));
-	  disp = mh_disposition (wh->draftfile);
-	}
-    }
+  if (!build_only)
+    disp = check_draft_disposition (wh, use_draft);
 
   switch (disp)
     {
@@ -221,7 +204,7 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
       break;  
     }
 
-  msgno = mh_msgset_first (msgset);
+  msgno = mh_msgset_first (msgset, RET_MSGNO);
   rc = mu_mailbox_get_message (mbox, msgno, &msg);
   if (rc)
     {
@@ -240,7 +223,6 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
   if (disp == DISP_REPLACE)
     {
       mu_stream_t str;
-      char *buf;
       
       rc = mu_file_stream_create (&str, wh->file,
 				  MU_STREAM_WRITE|MU_STREAM_CREAT);
@@ -251,6 +233,8 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
 	  exit (1);
 	}
 
+      mh_fvm_set_output (fvm, str);
+
       if (has_fcc)
 	{
 	  mu_message_t tmp_msg;
@@ -260,15 +244,13 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
 	  mu_message_create_copy (&tmp_msg, msg);
 	  mu_message_get_header (tmp_msg, &hdr);
 	  text = mu_opool_finish (fcc_pool, NULL);
-	  mu_header_set_value (hdr, MU_HEADER_FCC, text, 0);
-	  mh_format (&format, tmp_msg, msgno, width, &buf);
+	  mu_header_set_value (hdr, MU_HEADER_FCC, text, 1);
+	  mh_fvm_run (fvm, tmp_msg);
 	  mu_message_destroy (&tmp_msg, NULL);
 	}
       else
-	mh_format (&format, msg, msgno, width, &buf);
+	mh_fvm_run (fvm, msg);
       
-      mu_stream_write (str, buf, strlen (buf), NULL);
-
       if (mhl_filter)
 	{
 	  mu_list_t filter = mhl_format_compile (mhl_filter);
@@ -278,8 +260,8 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
 	  mhl_format_destroy (&filter);
 	}
 
+      mh_fvm_set_output (fvm, mu_strout);
       mu_stream_destroy (&str);
-      free (buf);
     }
 
   {
@@ -302,26 +284,34 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
   }
 }
 
+static struct mh_optinit optinit[] = {
+  { "draftfolder", "Draft-Folder" },
+  { "whatnowproc", "whatnowproc" },
+  { NULL }
+};
+
 int
 main (int argc, char **argv)
 {
   int rc;
 
-  draftfolder = mh_global_profile_get ("Draft-Folder", NULL);
-  whatnowproc = mh_global_profile_get ("whatnowproc", NULL);
-
-  mh_getopt (&argc, &argv, options, MH_GETOPT_DEFAULT_FOLDER,
-	     args_doc, prog_doc, NULL);
-
-  if (!format_str)
-    format_str = default_format_str;
-
-  if (mh_format_parse (format_str, &format))
+  mh_getopt_ext (&argc, &argv, options, MH_GETOPT_DEFAULT_FOLDER, optinit,
+		 args_doc, prog_doc, NULL);
+  if (!format)
     {
-      mu_error (_("Bad format string"));
-      exit (1);
+      if (mh_format_string_parse (&format, default_format_str,
+				  NULL, MH_FMT_PARSE_DEFAULT))
+	{
+	  mu_error (_("INTERNAL ERROR: bad built-in format; please report"));
+	  exit (1);
+	}
     }
 
+  mh_fvm_create (&fvm, 0);
+  mh_fvm_set_format (fvm, format);
+  mh_fvm_set_width (fvm, width ? width : mh_width ());
+  mh_format_destroy (&format);
+
   mbox = mh_open_folder (mh_current_folder (), MU_STREAM_RDWR);
   mh_msgset_parse (&msgset, mbox, argc, argv, "cur");
   if (!mh_msgset_single_message (msgset))
@@ -344,7 +334,7 @@ main (int argc, char **argv)
   make_draft (mbox, DISP_REPLACE, &wh_env);
 
   /* Exit immediately if --build is given */
-  if (build_only || nowhatnowproc)
+  if (build_only || wh_env.nowhatnowproc)
     return 0;
 
   rc = mh_whatnowproc (&wh_env, initial_edit, whatnowproc);
