@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2018 Free Software Foundation, Inc.
+   Copyright (C) 2010-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1124,7 +1124,7 @@ print_list_item (void *item, void *data)
 		    (resp->type & MU_FOLDER_ATTRIBUTE_FILE) ? 'f' : '-',
 		    resp->separator ?
 		        resp->separator : MU_HIERARCHY_DELIMITER,
-		    resp->level,
+		    resp->depth,
 		    resp->name);
   return 0;
 }
