@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2005, 2007-2012 Free Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,8 +16,9 @@
 
 #include "pop3d.h"
 #include "mailutils/pam.h"
-#include "mailutils/libargp.h"
+#include "mailutils/cli.h"
 #include "mailutils/pop3.h"
+#include "mailutils/kwd.h"
 #include "tcpwrap.h"
 
 mu_mailbox_t mbox;
@@ -29,18 +30,12 @@ mu_m_server_t server;
 unsigned int idle_timeout;
 int pop3d_transcript;
 int debug_mode;
-int tls_required;
 int pop3d_xlines;
 char *apop_database_name = APOP_PASSFILE;
 int apop_database_safety = MU_FILE_SAFETY_ALL;
 uid_t apop_database_owner;
 int apop_database_owner_set;
 
-#ifdef WITH_TLS
-int tls_available;
-int tls_done;
-#endif /* WITH_TLS */
-
 int initial_state = AUTHORIZATION; 
 
 /* Should all the messages be undeleted on startup */
@@ -54,24 +49,54 @@ char *login_stat_file = LOGIN_STAT_FILE;
 unsigned expire = EXPIRE_NEVER; /* Expire messages after this number of days */
 int expire_on_exit = 0;         /* Delete expired messages on exit */
 
-static error_t pop3d_parse_opt  (int key, char *arg, struct argp_state *astate);
-
 const char *program_version = "pop3d (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU pop3d -- the POP3 daemon.");
+
+static void
+set_foreground (struct mu_parseopt *po, struct mu_option *opt,
+		char const *arg)
+{
+  mu_m_server_set_foreground (server, 1);
+}
 
-#define OPT_FOREGROUND      256
-
-static struct argp_option options[] = {
-#define GRP 0
-  { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), GRP+1},
-  { "inetd",  'i', 0, 0, N_("run in inetd mode"), GRP+1},
-  { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
-    N_("runs in daemon mode with a maximum of NUMBER children"), GRP+1 },
-#undef GRP
-
-  {NULL, 0, NULL, 0, NULL, 0}
-};
+static void
+set_inetd_mode (struct mu_parseopt *po, struct mu_option *opt,
+		char const *arg)
+{
+  mu_m_server_set_mode (server, MODE_INTERACTIVE);
+}
+  
+static void
+set_daemon_mode (struct mu_parseopt *po, struct mu_option *opt,
+		 char const *arg)
+{
+  mu_m_server_set_mode (server, MODE_DAEMON);
+  if (arg)
+    {
+      size_t max_children;
+      char *errmsg;
+      int rc = mu_str_to_c (arg, mu_c_size, &max_children, &errmsg);
+      if (rc)
+	{
+	  mu_parseopt_error (po, _("%s: bad argument"), arg);
+	  exit (po->po_exit_error);
+	}
+      mu_m_server_set_max_children (server, max_children);
+    }
+}
 
+static struct mu_option pop3d_options[] = {
+  { "foreground",  0, NULL, MU_OPTION_DEFAULT,
+    N_("remain in foreground"),
+    mu_c_bool, NULL, set_foreground },
+  { "inetd",  'i', NULL, MU_OPTION_DEFAULT,
+    N_("run in inetd mode"),
+    mu_c_bool, NULL, set_inetd_mode },
+  { "daemon", 'd', N_("NUMBER"), MU_OPTION_ARG_OPTIONAL,
+    N_("runs in daemon mode with a maximum of NUMBER children"),
+    mu_c_string, NULL, set_daemon_mode },
+  MU_OPTION_END
+}, *options[] = { pop3d_options, NULL };
+
 static int
 cb_bulletin_source (void *data, mu_config_value_t *val)
 {
@@ -134,33 +159,61 @@ cb_bulletin_db (void *data, mu_config_value_t *val)
 }
 #endif
 
-struct pop3d_srv_config
+static int
+cb_tls (void *data, mu_config_value_t *val)
 {
-  struct mu_srv_config m_cfg;
-  int tls;
-};
+  int *res = data;
+  static struct mu_kwd tls_kwd[] = {
+    { "no", tls_no },
+    { "false", tls_no },
+    { "off", tls_no },
+    { "0", tls_no },
+    { "ondemand", tls_ondemand },
+    { "stls", tls_ondemand },
+    { "required", tls_required },
+    { "connection", tls_connection },
+    /* For compatibility with prior versions: */
+    { "yes", tls_connection }, 
+    { "true", tls_connection },
+    { "on", tls_connection },
+    { "1", tls_connection },
+    { NULL }
+  };
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+
+  if (mu_kwd_xlat_name (tls_kwd, val->v.string, res))
+    mu_error (_("not a valid tls keyword: %s"), val->v.string);
+  return 0;
+}
 
 static struct mu_cfg_param pop3d_srv_param[] = {
-  { "tls", mu_cfg_bool, NULL, mu_offsetof (struct pop3d_srv_config, tls), NULL,
-    N_("Use TLS encryption for this server")
-  },
+  { "tls-mode", mu_cfg_callback,
+    NULL, mu_offsetof (struct pop3d_srv_config, tls_mode), cb_tls,
+    N_("Kind of TLS encryption to use for this server"),
+    /* TRANSLATORS: words to the right of : are keywords - do not translate */
+    N_("arg: false|true|ondemand|stls|requred|connection") },
+  { "tls", mu_cfg_section,
+    NULL, mu_offsetof (struct pop3d_srv_config, tls_conf) },
   { NULL }
 };
     
 static struct mu_cfg_param pop3d_cfg_param[] = {
-  { "undelete", mu_cfg_bool, &undelete_on_startup, 0, NULL,
+  { "undelete", mu_c_bool, &undelete_on_startup, 0, NULL,
     N_("On startup, clear deletion marks from all the messages.") },
-  { "expire", mu_cfg_uint, &expire, 0, NULL,
+  { "expire", mu_c_uint, &expire, 0, NULL,
     N_("Automatically expire read messages after the given number of days."),
     N_("days") },
-  { "delete-expired", mu_cfg_bool, &expire_on_exit, 0, NULL,
+  { "delete-expired", mu_c_bool, &expire_on_exit, 0, NULL,
     N_("Delete expired messages upon closing the mailbox.") },
-  { "scan-lines", mu_cfg_bool, &pop3d_xlines, 0, NULL,
+  { "scan-lines", mu_c_bool, &pop3d_xlines, 0, NULL,
     N_("Output the number of lines in the message in its scan listing.") },
-  { "apop-database-file", mu_cfg_string, &apop_database_name, 0, NULL,
+  { "apop-database-file", mu_c_string, &apop_database_name, 0, NULL,
     N_("set APOP database file name or URL") },
   { "apop-database-owner", mu_cfg_callback, NULL, 0, cb_apop_database_owner,
-    N_("Name or UID of the APOP database owner") },
+    N_("Name or UID of the APOP database owner"),
+    N_("arg: string") },
   { "apop-database-safety", mu_cfg_callback, NULL, 0, cb_apop_safety_checks,
     N_("Configure safety checks for APOP database files.  Argument is a list or "
        "sequence of check names optionally prefixed with '+' to enable or "
@@ -177,47 +230,41 @@ static struct mu_cfg_param pop3d_cfg_param[] = {
        "  awrdir        forbid files in world writable directories\n"),
     N_("arg: list") },  
     
-#ifdef WITH_TLS
-  { "tls-required", mu_cfg_bool, &tls_required, 0, NULL,
-     N_("Always require STLS before entering authentication phase.") },
-#endif
+  { "tls", mu_cfg_section, &global_tls_conf },
+  { "tls-mode", mu_cfg_callback,
+    &global_tls_mode, 0, cb_tls,
+    N_("Kind of TLS encryption to use for the inetd server"
+       " and all server blocks that lack the tls-mode statement."),
+    /* TRANSLATORS: words to the right of : are keywords - do not translate */
+    N_("arg: false|true|ondemand|stls|requred|connection") },
+  
 #ifdef ENABLE_LOGIN_DELAY
-  { "login-delay", mu_cfg_time, &login_delay, 0, NULL,
+  { "login-delay", mu_c_time, &login_delay, 0, NULL,
     N_("Set the minimal allowed delay between two successive logins.") },
-  { "stat-file", mu_cfg_string, &login_stat_file, 0, NULL,
+  { "stat-file", mu_c_string, &login_stat_file, 0, NULL,
     N_("Set the name of login statistics file (for login-delay).") },
 #endif
   { "bulletin-source", mu_cfg_callback, NULL, 0, cb_bulletin_source,
     N_("Get bulletins from the specified mailbox."),
-    N_("url") },
+    N_("url: string") },
 #ifdef ENABLE_DBM
   { "bulletin-db", mu_cfg_callback, NULL, 0, cb_bulletin_db,
     N_("Set the bulletin database file name."),
-    N_("file") },
+    N_("file: string") },
 #endif
-  { "output-buffer-size", mu_cfg_size, &pop3d_output_bufsize, 0, NULL,
+  { "output-buffer-size", mu_c_size, &pop3d_output_bufsize, 0, NULL,
     N_("Size of the output buffer.") },
   { "mandatory-locking", mu_cfg_section },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("Server configuration.") },
-  { "transcript", mu_cfg_bool, &pop3d_transcript, 0, NULL,
+  { "transcript", mu_c_bool, &pop3d_transcript, 0, NULL,
     N_("Set global transcript mode.") },
   TCP_WRAPPERS_CONFIG
   { NULL }
 };
     
-static struct argp argp = {
-  options,
-  pop3d_parse_opt,
-  NULL,
-  doc,
-  NULL,
-  NULL, NULL
-};
-
-static const char *pop3d_argp_capa[] = {
+static char *capa[] = {
   "auth",
-  "common",
   "debug",
   "mailbox",
   "locking",
@@ -225,41 +272,13 @@ static const char *pop3d_argp_capa[] = {
   NULL
 };
 
-static error_t
-pop3d_parse_opt (int key, char *arg, struct argp_state *astate)
-{
-  static mu_list_t lst;
-  
-  switch (key)
-    {
-    case 'd':
-      mu_argp_node_list_new (lst, "mode", "daemon");
-      if (arg)
-	mu_argp_node_list_new (lst, "max-children", arg);
-      break;
-
-    case 'i':
-      mu_argp_node_list_new (lst, "mode", "inetd");
-      break;
-
-    case OPT_FOREGROUND:
-      mu_argp_node_list_new (lst, "foreground", "yes");
-      break;
-      
-    case ARGP_KEY_INIT:
-      mu_argp_node_list_init (&lst);
-      break;
-      
-    case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (lst, NULL, NULL);
-      break;
-      
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
-}
-
+struct mu_cli_setup cli = {
+  .optv = options,
+  .cfg = pop3d_cfg_param,
+  .prog_doc = N_("GNU pop3d -- the POP3 daemon."),
+  .server = 1
+};
+
 int
 pop3d_get_client_address (int fd, struct sockaddr_in *pcs)
 {
@@ -292,19 +311,30 @@ pop3d_get_client_address (int fd, struct sockaddr_in *pcs)
       ofd       --  output descriptor
       tls       --  initiate encrypted connection */
 int
-pop3d_mainloop (int ifd, int ofd, int tls)
+pop3d_mainloop (int ifd, int ofd, struct pop3d_srv_config *cfg)
 {
   int status = OK;
   char buffer[512];
   static int sigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP, SIGPIPE,
 			  SIGABRT, SIGINT, SIGQUIT, SIGTERM, SIGHUP, SIGALRM };
-
+  struct pop3d_session session;
+     
   mu_set_signals (pop3d_child_signal, sigtab, MU_ARRAY_SIZE (sigtab));
 
-  pop3d_setio (ifd, ofd, tls);
+  pop3d_setio (ifd, ofd,
+	       cfg->tls_mode == tls_connection ? &cfg->tls_conf : NULL);
 
-  state = tls ? AUTHORIZATION : initial_state;
+  if (cfg->tls_mode == tls_required)
+    initial_state = INITIAL;
+  
+  state = cfg->tls_mode == tls_connection ? AUTHORIZATION : initial_state;
 
+  pop3d_session_init (&session);
+  session.tls_mode = cfg->tls_mode;
+  session.tls_conf = &cfg->tls_conf;
+  
+  /* FIXME: state should also be in the session? */
+  
   /* Prepare the shared secret for APOP.  */
   {
     char *local_hostname;
@@ -353,7 +383,7 @@ pop3d_mainloop (int ifd, int ofd, int tls)
       manlock_touchlock (mbox);
 
       if ((handler = pop3d_find_command (cmd)) != NULL)
-	status = handler (arg);
+	status = handler (arg, &session);
       else
 	status = ERR_BAD_CMD;
 
@@ -361,95 +391,24 @@ pop3d_mainloop (int ifd, int ofd, int tls)
 	pop3d_outf ("-ERR %s\n", pop3d_error_string (status));
     }
 
+  pop3d_session_free (&session);
+  
   pop3d_bye ();
 
   return status;
 }
-
-static int
-set_strerr_flt ()
-{
-  mu_stream_t flt, trans[2];
-  int rc;
-  
-  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
-  if (rc == 0)
-    {
-      char sessidstr[10];
-      char *argv[] = { "inline-comment", NULL, "-S", NULL };
-
-      snprintf (sessidstr, sizeof sessidstr, "%08lx:", mu_session_id);
-      argv[1] = sessidstr;
-      rc = mu_filter_create_args (&flt, trans[0], "inline-comment", 3,
-				  (const char **)argv,
-				  MU_FILTER_ENCODE, MU_STREAM_WRITE);
-      mu_stream_unref (trans[0]);
-      if (rc == 0)
-	{
-	  mu_stream_set_buffer (flt, mu_buffer_line, 0);
-	  trans[0] = flt;
-	  trans[1] = NULL;
-	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM,
-				MU_IOCTL_OP_SET, trans);
-	  mu_stream_unref (trans[0]);
-	  if (rc)
-	    mu_error (_("%s failed: %s"), "MU_IOCTL_SET_STREAM",
-		      mu_stream_strerror (mu_strerr, rc));
-	}
-      else
-	mu_error (_("cannot create log filter stream: %s"), mu_strerror (rc));
-    }
-  else
-    {
-      mu_error (_("%s failed: %s"), "MU_IOCTL_GET_STREAM",
-		mu_stream_strerror (mu_strerr, rc));
-    }
-  return rc;
-}
-
-static void
-clr_strerr_flt ()
-{
-  mu_stream_t flt, trans[2];
-  int rc;
-
-  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
-  if (rc == 0)
-    {
-      flt = trans[0];
-
-      rc = mu_stream_ioctl (flt, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
-      if (rc == 0)
-	{
-	  mu_stream_unref (trans[0]);
-	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM,
-				MU_IOCTL_OP_SET, trans);
-	  if (rc == 0)
-	    mu_stream_unref (flt);
-	}
-    }
-}
-
+
 int
 pop3d_connection (int fd, struct sockaddr *sa, int salen,
 		  struct mu_srv_config *pconf,
 		  void *data)
 {
   struct pop3d_srv_config *cfg = (struct pop3d_srv_config *) pconf;
-  int rc;
   
-  idle_timeout = pconf->timeout;
-  pop3d_transcript = pconf->transcript;
+  idle_timeout = cfg->m_cfg.timeout;
+  pop3d_transcript = cfg->m_cfg.transcript;
 
-  if (mu_log_session_id)
-    rc = set_strerr_flt ();
-  else
-    rc = 1;
-  
-  pop3d_mainloop (fd, fd, cfg->tls);
-
-  if (rc == 0)
-    clr_strerr_flt ();
+  pop3d_mainloop (fd, fd, cfg);
   return 0;
 }
 
@@ -483,16 +442,11 @@ main (int argc, char **argv)
   /* Register the desired formats.  */
   mu_register_local_mbox_formats ();
 
-#ifdef WITH_TLS
-  mu_gocs_register ("tls", mu_tls_module_init);
-#endif /* WITH_TLS */
   mu_tcpwrapper_cfg_init ();
   manlock_cfg_init ();
   mu_acl_cfg_init ();
-  mu_m_server_cfg_init (pop3d_srv_param);
+  mu_tls_cfg_init ();
   
-  mu_argp_init (NULL, NULL);
-  	
   mu_m_server_create (&server, program_version);
   mu_m_server_set_config_size (server, sizeof (struct pop3d_srv_config));
   mu_m_server_set_conn (server, pop3d_connection);
@@ -503,6 +457,7 @@ main (int argc, char **argv)
   mu_m_server_set_default_port (server, 110);
   mu_m_server_set_timeout (server, 600);
   mu_m_server_set_strexit (server, mu_strexit);
+  mu_m_server_cfg_init (server, pop3d_srv_param);
 
   mu_alloc_die_hook = pop3d_alloc_die;
 
@@ -512,14 +467,14 @@ main (int argc, char **argv)
 #ifdef ENABLE_DBM
   set_dbm_safety ();
 #endif
-  
-  if (mu_app_init (&argp, pop3d_argp_capa, pop3d_cfg_param, 
-		   argc, argv, 0, NULL, server))
-    exit (EX_CONFIG); /* FIXME: No way to discern from EX_USAGE? */
 
-  if (tls_required)
-    initial_state = INITIAL;
-  
+  mu_cli (argc, argv, &cli, capa, server, &argc, &argv);
+  if (argc)
+    {
+      mu_error (_("too many arguments"));
+      exit (EX_USAGE);
+    }
+
   if (expire == 0)
     expire_on_exit = 1;
 
@@ -567,15 +522,7 @@ main (int argc, char **argv)
   umask (S_IROTH | S_IWOTH | S_IXOTH);	/* 007 */
 
   /* Check TLS environment, i.e. cert and key files */
-#ifdef WITH_TLS
-  tls_available = mu_check_tls_environment ();
-  if (tls_available)
-    {
-      tls_available = mu_init_tls_libs (1);
-      if (tls_available)
-	enable_stls ();
-    }
-#endif /* WITH_TLS */
+  mu_m_server_set_preflight (server, stls_preflight);
 
   /* Actually run the daemon.  */
   if (mu_m_server_mode (server) == MODE_DAEMON)
@@ -587,9 +534,35 @@ main (int argc, char **argv)
     }
   else
     {
+      struct pop3d_srv_config cfg;
+      memset (&cfg, 0, sizeof cfg);
+
+      switch (stls_server_check (&cfg, "<inetd>"))
+	{
+	case MU_TLS_CONFIG_OK:
+	  if (mu_init_tls_libs ())
+	    status = EX_OK;
+	  else
+	    {
+	      mu_error (_("TLS is not configured, but requested in the "
+			  "configuration"));
+	      exit (EX_CONFIG);
+	    }
+	  break;
+
+	case MU_TLS_CONFIG_NULL:
+	  break;
+
+	case MU_TLS_CONFIG_UNSAFE:
+	  exit (EX_CONFIG);
+
+	default:
+	  exit (EX_UNAVAILABLE);
+	}
+
       /* Make sure we are in the root directory.  */
       chdir ("/");
-      status = pop3d_mainloop (MU_STDIN_FD, MU_STDOUT_FD, 0);
+      status = pop3d_mainloop (MU_STDIN_FD, MU_STDOUT_FD, &cfg);
     }
   
   if (status)
